@@ -7,9 +7,9 @@ from typing import *
 import yaml
 from networkx import DiGraph, ancestors
 
-from duckietown_challenges.cmd_submit_build import parse_complete_tag
 from . import dclogger
 from .challenges_constants import ChallengesConstants
+from .cmd_submit_build import parse_complete_tag
 from .exceptions import InvalidConfiguration
 from .utils import indent, safe_yaml_dump, check_isinstance, wrap_config_reader2
 
@@ -26,7 +26,7 @@ STATE_FAILED = 'FAILED'
 
 ALLOWED_CONDITION_TRIGGERS = ChallengesConstants.ALLOWED_JOB_STATUS
 
-allowed_permissions = ['snoop', 'change', 'moderate', 'grant']
+# allowed_permissions = ['snoop', 'change', 'moderate', 'grant']
 
 
 @dataclass(repr=False)
@@ -260,6 +260,7 @@ class ChallengeStep:
     evaluation_parameters: EvaluationParameters
     features_required: Dict[str, Any]
     timeout: float
+    uptodate_token: Optional[str] = None
 
     def as_dict(self):
         data = {}
@@ -268,6 +269,7 @@ class ChallengeStep:
         data['evaluation_parameters'] = self.evaluation_parameters.as_dict()
         data['features_required'] = self.features_required
         data['timeout'] = self.timeout
+        data['uptodate_token'] = self.uptodate_token
         return data
 
     # noinspection PyArgumentList
@@ -279,9 +281,9 @@ class ChallengeStep:
         evaluation_parameters = EvaluationParameters.from_yaml(data.pop('evaluation_parameters'))
         features_required = data.pop('features_required', {})
         timeout = data.pop('timeout')
-
+        uptodate_token = data.pop('uptodate_token', None)
         return ChallengeStep(name, title, description, evaluation_parameters,
-                             features_required, timeout=timeout)
+                             features_required, timeout=timeout, uptodate_token=uptodate_token)
 
 
 class NotEquivalent(Exception):
@@ -400,14 +402,16 @@ class ChallengeTransitions:
                 msg = 'Ignoring invalid step %s -> %s' % (k, ks)
                 dclogger.error(msg)
                 status.pop(k)
+
             if ks not in ChallengesConstants.ALLOWED_JOB_STATUS:
-                msg = 'Ignoring invalid step %s -> %s' % (k, ks)
+                msg = f'Ignoring invalid step {k} -> {ks!r}'
                 dclogger.error(msg)
                 status.pop(k)
 
             # timeout or aborted or host error = like it never happened
             if ks in [CS.STATUS_JOB_TIMEOUT, CS.STATUS_JOB_ABORTED, CS.STATUS_JOB_HOST_ERROR]:
-                status.pop(k)
+                if k in status:
+                    status.pop(k)
 
         # make sure that the steps in which they depend are ok
 
@@ -557,12 +561,12 @@ class ChallengeDescription:
     date_open: datetime
     date_close: datetime
     steps: Dict[str, ChallengeStep]
-    roles: Any
+    # roles: Any
     ct: ChallengeTransitions
     scoring: Scoring
 
     def __init__(self, name, title, description, protocol,
-                 date_open, date_close, steps, roles, transitions, tags, scoring):
+                 date_open, date_close, steps, transitions, tags, scoring):
         self.name = name
         self.title = title
         self.scoring = scoring
@@ -573,19 +577,19 @@ class ChallengeDescription:
         check_isinstance(date_close, datetime)
         self.date_close = date_close
         self.steps = steps
-        self.roles = roles
+        # self.roles = roles
         self.tags = tags
 
-        for k, permissions in self.roles.items():
-            if not k.startswith('user:'):
-                msg = 'Permissions should start with "user:", %s' % k
-                raise InvalidChallengeDescription(msg)
-            p2 = dict(**permissions)
-            for perm in allowed_permissions:
-                p2.pop(perm, None)
-            if p2:
-                msg = 'Unknown permissions: %s' % p2
-                raise InvalidChallengeDescription(msg)
+        # for k, permissions in self.roles.items():
+        #     if not k.startswith('user:'):
+        #         msg = 'Permissions should start with "user:", %s' % k
+        #         raise InvalidChallengeDescription(msg)
+        #     p2 = dict(**permissions)
+        #     for perm in allowed_permissions:
+        #         p2.pop(perm, None)
+        #     if p2:
+        #         msg = 'Unknown permissions: %s' % p2
+        #         raise InvalidChallengeDescription(msg)
 
         self.first_step = None
         self.ct = ChallengeTransitions.from_steps_transitions(list(self.steps), transitions)
@@ -608,18 +612,32 @@ class ChallengeDescription:
         date_open = interpret_date(data.pop('date-open'))
         date_close = interpret_date(data.pop('date-close'))
 
-        roles = data.pop('roles')
-        transitions = data.pop('transitions')
+        data.pop('roles', None)
+
         steps = data.pop('steps')
         Steps = {}
         for k, v in steps.items():
             Steps[k] = ChallengeStep.from_yaml(v, k)
 
+        transitions = data.pop('transitions', None)
+        if transitions is None:
+            if len(Steps) == 1:
+                stepname = list(Steps)[0]
+                transitions = [
+                    ['START', 'success', stepname, ],
+                    [stepname, 'success', 'SUCCESS'],
+                    [stepname, 'failed', 'FAILED'],
+                    [stepname, 'error', 'ERROR'],
+                ]
+            else:
+                msg = 'Need transitions if there is more than one step.'
+                raise ValueError(msg)
+
         scoring = Scoring.from_yaml(data.pop('scoring'))
 
         return ChallengeDescription(name, title, description,
                                     protocol, date_open, date_close, Steps,
-                                    roles=roles, transitions=transitions,
+                                    transitions=transitions,
                                     tags=tags, scoring=scoring)
 
     def as_dict(self):
@@ -630,14 +648,16 @@ class ChallengeDescription:
         data['protocol'] = self.protocol
         data['date-open'] = self.date_open.isoformat() if self.date_open else None
         data['date-close'] = self.date_close.isoformat() if self.date_close else None
-        data['roles'] = self.roles
+        # data['roles'] = self.roles
         data['transitions'] = []
         for t in self.ct.transitions:
             tt = [t.first, t.condition, t.second]
             data['transitions'].append(tt)
-        data['steps'] = {}
+        steps = {}
         for k, v in self.steps.items():
-            data['steps'][k] = v.as_dict()
+            steps[k] = v.as_dict()
+        data['steps'] = steps
+
         data['tags'] = self.tags
         data['scoring'] = self.scoring.as_dict()
         return data
@@ -665,17 +685,26 @@ def interpret_date(d):
 
 @dataclass(repr=False)
 class SubmissionDescription:
-    challenge_names: List[str]
-    protocol: List[str]
+    challenge_names: Optional[List[str]]
+    protocols: List[str]
     user_label: Optional[str]
     user_metadata: dict
     description: Optional[str]
 
+    def __post_init_(self):
+        if self.challenge_names is not None:
+            if not isinstance(self.challenge_names, list):
+                msg = 'Expected a list of strings for challenge names, got %s' % self.challenge_names
+                raise ValueError(msg)
+        if not isinstance(self.protocols, list):
+            msg = 'Expected a list of strings for protocols names, got %s' % self.protocols
+            raise ValueError(msg)
+
     def __repr__(self):
         return nice_repr(self)
 
     def as_dict(self):
-        return dict(protocol=self.protocol,
+        return dict(protocols=self.protocols,
                     challenge_names=self.challenge_names,
                     user_label=self.user_label,
                     user_metadata=self.user_metadata,
@@ -685,19 +714,27 @@ class SubmissionDescription:
     @classmethod
     @wrap_config_reader2
     def from_yaml(cls, data):
-        challenge_name = data.pop('challenge')
-        if isinstance(challenge_name, list):
-            challenges = challenge_name
+        challenge_name = data.pop('challenge', None)
+        if challenge_name is None:
+            challenges = None
         else:
-            challenges = [challenge_name]
+            if isinstance(challenge_name, list):
+                challenges = challenge_name
+            else:
+                challenges = [challenge_name]
 
         protocol = data.pop('protocol')
+        if isinstance(protocol, list):
+            protocols = protocol
+        else:
+            protocols = [protocol]
+
         description = data.pop('description', None)
         user_label = data.pop('user-label', None)
         user_metadata = data.pop('user-payload', None)
 
         return SubmissionDescription(challenge_names=challenges,
-                                     protocol=protocol,
+                                     protocols=protocols,
                                      description=description,
                                      user_label=user_label,
                                      user_metadata=user_metadata)
