@@ -19,7 +19,7 @@ class ChallengesConstants:
         STATUS_JOB_HOST_ERROR
     ]
 
-    JOB_TIMEOUT_MINUTES = 30
+    # JOB_TIMEOUT_MINUTES = 30
 
     DEFAULT_DTSERVER = 'https://challenges.duckietown.org/v4'
     DEFAULT_TIMEOUT = 5
@@ -32,7 +32,10 @@ class ChallengesConstants:
         components = '/api/components'
         challenges = '/api/challenges'
         take_submission = '/api/take-submission'
+        auth = '/api/auth'
         reset_submission = '/api/reset-submission'
+        change_user_priority = '/api/change-user-priority'
+        change_admin_priority = '/api/change-admin-priority'
         reset_job = '/api/reset-job'
         submissions_list = '/api/submissions-list'
         submission_single = '/api/submission-single'
