@@ -11,6 +11,7 @@ import traceback
 from collections import namedtuple
 from contextlib import contextmanager
 from dataclasses import dataclass
+from typing import *
 
 from duckietown_challenges import ChallengesConstants
 from . import dclogger, ENV_CHALLENGE_STEP_NAME
@@ -21,7 +22,7 @@ from .exceptions import InvalidSubmission, InvalidEvaluator, InvalidEnvironment
 from .solution_interface import ChallengeInterfaceSolution, ChallengeInterfaceEvaluator
 from .utils import d8n_make_sure_dir_exists
 from .yaml_utils import read_yaml_file, write_yaml
-from typing import *
+
 
 @dataclass
 class ChallengeFile:
@@ -30,11 +31,11 @@ class ChallengeFile:
     contents: Optional[bytes]
     description: str
 
+
 # ChallengeFile = namedtuple('ChallengeFile', 'basename from_file contents description')
 ReportedScore = namedtuple('ReportedScore', 'name value description')
 
 
-
 def check_valid_basename(s):
     pass  # TODO
 
@@ -255,6 +256,7 @@ def a_tmp_dir(root):
 
 
 class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
+    ipfs_hashes: Dict[str, str]
 
     def __init__(self, root=DEFAULT_ROOT):
         dclogger.info(f'ChallengeInterfaceEvaluatorConcrete root = {root}')
@@ -264,6 +266,7 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
         self.parameters = None
 
         self.evaluation_files = FS()  # -> ChallengeFile
+        self.ipfs_hashes = {}
         self.scores = {}  # str -> ReportedScore
 
     def set_challenge_parameters(self, data):
@@ -303,6 +306,8 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
 
     def get_solution_output_files(self):
         d = os.path.join(self.root, CHALLENGE_SOLUTION_OUTPUT_DIR)
+        if not os.path.exists(d):
+            return []
         fns = list(os.listdir(d))
         return fns
 
@@ -328,6 +333,16 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
 
         self.scores[name] = ReportedScore(name, value, description)
 
+    def set_evaluation_ipfs_hash(self, rpath: str, ipfs_hash: str):
+
+        if not isinstance(ipfs_hash, str):
+            raise ValueError(ipfs_hash)
+        ipfs_hash = ipfs_hash.strip()
+        if not ipfs_hash.startswith('Qm'):
+            raise ValueError(ipfs_hash)
+        rpath = os.path.join(CHALLENGE_EVALUATION_OUTPUT_DIR, rpath)
+        self.ipfs_hashes[rpath] = ipfs_hash
+
     def set_evaluation_file(self, basename, from_file, description=None):
         try:
             self.evaluation_files.add(basename, from_file, description)
@@ -377,7 +392,7 @@ class ChallengeInterfaceEvaluatorConcrete(ChallengeInterfaceEvaluator):
         scores = {}
         for k, v in self.scores.items():
             scores[k] = v.value
-        cr = ChallengeResults(status, msg, scores)
+        cr = ChallengeResults(status, msg, scores, ipfs_hashes=self.ipfs_hashes)
 
         declare_challenge_results(self.root, cr)
 
@@ -447,6 +462,7 @@ def get_completed_step_evaluation_files(root, step_name):
         msg = 'Could not find dir %s' % d
         raise InvalidEnvironment(msg)
 
+    dclogger.info('step dir is %s' % d)
     return list(os.listdir(d))
 
 
@@ -457,10 +473,18 @@ def get_completed_step_evaluation_file(root, step_name, basename):
     #     msg = 'No file %r' % basename
     #     raise KeyError(msg)
 
-    fn = os.path.join(root, CHALLENGE_PREVIOUS_STEPS_DIR, step_name, CHALLENGE_EVALUATION_OUTPUT_DIR, basename)
+    step_dir = os.path.join(root, CHALLENGE_PREVIOUS_STEPS_DIR, step_name, CHALLENGE_EVALUATION_OUTPUT_DIR)
+    fn = os.path.join(step_dir, basename)
     if not os.path.exists(fn):
-        msg = 'Cannot find %s; know %s' % (fn, available)
-        raise KeyError(msg)
+        msg = 'File %s -> %s does not exist ' % (basename, fn)
+
+        for x in available:
+            msg += f'\n available {x}'
+
+        msg += f'\n\n step_dir: {step_dir}'
+        msg += '\n\n list: ' + str(os.listdir(step_dir))
+        dclogger.warning(msg)
+        # raise KeyError(msg)
     return fn
 
 
@@ -492,6 +516,7 @@ def wrap_evaluator(evaluator, root=DEFAULT_ROOT):
 
     cie = ChallengeInterfaceEvaluatorConcrete(root=root)
 
+    # noinspection PyBroadException
     try:
         try:
             evaluator.prepare(cie)
@@ -536,7 +561,7 @@ def wrap_evaluator(evaluator, root=DEFAULT_ROOT):
         msg = 'InvalidEnvironment:\n%s' % traceback.format_exc()
         declare(ChallengesConstants.STATUS_JOB_HOST_ERROR, msg)
 
-    except BaseException as e:
+    except BaseException:
         msg = 'Unexpected exception:\n%s' % traceback.format_exc()
         declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
 
@@ -583,55 +608,60 @@ def wrap_scorer(evaluator, root=DEFAULT_ROOT):
         declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
 
 
-
-
 @contextmanager
 def scoring_context(root=DEFAULT_ROOT) -> ContextManager[ChallengeInterfaceEvaluator]:
     dclogger.info('Environment variables:\n\n' + json.dumps(dict(os.environ), indent=2))
 
-
     from .col_logging import setup_logging_color
     setup_logging_color()
 
-    def declare(status, message, scores=None):
+    def declare(status, message, scores: Dict, ipfs_hashes: Dict[str, str]):
+        # write files
+        d = os.path.join(root, CHALLENGE_EVALUATION_OUTPUT_DIR)
+        cie.evaluation_files.write(d)
+
         if status != ChallengesConstants.STATUS_JOB_SUCCESS:
             msg = 'declare %s:\n%s' % (status, message)
             dclogger.error(msg)
         else:
             dclogger.info('Completed.')
-        cr = ChallengeResults(status, message, scores or {})
+        stats = {}
+        cr = ChallengeResults(status, message, scores=scores, stats=stats, ipfs_hashes=ipfs_hashes)
         declare_challenge_results(root, cr)
         retcode = 0 if status == ChallengesConstants.STATUS_JOB_SUCCESS else 1
         sys.exit(retcode)
 
     cie = ChallengeInterfaceEvaluatorConcrete(root=root)
 
-    try:
-        yield cie
-
+    def read_scores():
         scores = {}
         for k, v in cie.scores.items():
             scores[k] = v.value
-        declare(ChallengesConstants.STATUS_JOB_SUCCESS, None, scores)
+        return scores
+
+    try:
+        yield cie
+
+        declare(ChallengesConstants.STATUS_JOB_SUCCESS, None, read_scores(), cie.ipfs_hashes)
     # failure
     except InvalidSubmission:
         msg = 'InvalidSubmission:\n%s' % traceback.format_exc()
-        declare(ChallengesConstants.STATUS_JOB_FAILED, msg)
+        declare(ChallengesConstants.STATUS_JOB_FAILED, msg, read_scores(), cie.ipfs_hashes)
 
     except InvalidEvaluator:
         msg = 'InvalidEvaluator:\n%s' % traceback.format_exc()
-        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg, read_scores(), cie.ipfs_hashes)
 
     except InvalidEnvironment:
         msg = 'InvalidEnvironment:\n%s' % traceback.format_exc()
-        declare(ChallengesConstants.STATUS_JOB_HOST_ERROR, msg)
+        declare(ChallengesConstants.STATUS_JOB_HOST_ERROR, msg, read_scores(), cie.ipfs_hashes)
 
     except SystemExit:
         raise
 
     except BaseException:
         msg = 'Unexpected exception:\n%s' % traceback.format_exc()
-        declare(ChallengesConstants.STATUS_JOB_ERROR, msg)
+        declare(ChallengesConstants.STATUS_JOB_ERROR, msg, read_scores(), cie.ipfs_hashes)
 
 
 def wrap_solution(solution, root=DEFAULT_ROOT):
