@@ -81,7 +81,7 @@ class ChallengeInterfaceSolution(metaclass=ABCMeta):
     # Artefacts methods - saving files
 
     @abstractmethod
-    def set_solution_output_file(self, basename, from_file, description=None):
+    def set_solution_output_file(self, basename: str, from_file: str, description=None):
         """
             Creates an artefact called "basename" from the file `from_file`.
 
@@ -92,7 +92,7 @@ class ChallengeInterfaceSolution(metaclass=ABCMeta):
         """
 
     @abstractmethod
-    def set_solution_output_file_from_data(self, basename, contents, description=None):
+    def set_solution_output_file_from_data(self, basename: str, contents: bytes, description=None):
         """
             Same as before, but the contents is passed as a string.
 
@@ -121,7 +121,7 @@ class ChallengeInterfaceSolution(metaclass=ABCMeta):
         """
 
     @abstractmethod
-    def get_completed_step_solution_files(self, step_name):
+    def get_completed_step_solution_files(self, step_name: str):
         """
 
             Returns a list of names for the files completed in a previous step.
@@ -132,7 +132,7 @@ class ChallengeInterfaceSolution(metaclass=ABCMeta):
         """
 
     @abstractmethod
-    def get_completed_step_solution_file(self, step_name, basename):
+    def get_completed_step_solution_file(self, step_name: str, basename: str):
         """
 
             Returns a filename for one of the files completed in a previous step.
@@ -143,7 +143,7 @@ class ChallengeInterfaceSolution(metaclass=ABCMeta):
 
         """
 
-    def get_completed_step_solution_file_contents(self, step_name, basename):
+    def get_completed_step_solution_file_contents(self, step_name: str, basename: str) -> bytes:
         """
             Same as `get_completed_step_solution_file` but returns the contents
             directly.
@@ -154,7 +154,7 @@ class ChallengeInterfaceSolution(metaclass=ABCMeta):
 
         """
         fn = self.get_completed_step_solution_file(step_name, basename)
-        with open(fn) as f:
+        with open(fn, 'rb') as f:
             return f.read()
 
 
@@ -179,9 +179,9 @@ class ChallengeInterfaceEvaluator(metaclass=ABCMeta):
     def get_completed_step_evaluation_file(self, step_name, basename):
         """ Returns a filename for one of the files completed in a previous step."""
 
-    def get_completed_step_evaluation_file_contents(self, step_name, basename):
+    def get_completed_step_evaluation_file_contents(self, step_name, basename) -> bytes:
         fn = self.get_completed_step_evaluation_file(step_name, basename)
-        with open(fn) as f:
+        with open(fn, 'rb') as f:
             return f.read()
 
     @abstractmethod
@@ -233,6 +233,10 @@ class ChallengeInterfaceEvaluator(metaclass=ABCMeta):
             else:
                 self.set_evaluation_file(os.path.join(basename, bn), fn)
 
+    @abstractmethod
+    def set_evaluation_ipfs_hash(self, rpath: str, cid: str):
+        pass
+
     @abstractmethod
     def set_evaluation_file_from_data(self, basename, contents: bytes, description=None):
         pass
