@@ -2,6 +2,7 @@
 import json
 import math
 import os
+import re
 
 import decorator
 
@@ -264,3 +265,45 @@ def tag_from_date(d):
     s = s.replace('-', '_')
     s = s[:s.index('.')]
     return s
+
+
+def remove_escapes(s):
+    escape = re.compile('\x1b\[..?m')
+    return escape.sub("", s)
+
+
+def get_length_on_screen(s):
+    """ Returns the length of s without the escapes """
+    return len(remove_escapes(s))
+
+
+def pad_to_screen_length(s, desired_screen_length,
+                         pad=" ", last=None,
+                         align_right=False):
+    """
+        Pads a string so that it will appear of the given size
+        on the terminal.
+
+        align_right: aligns right instead of left (default)
+    """
+    assert isinstance(desired_screen_length, int)
+    # todo: assert pad = 1
+    current_size = get_length_on_screen(s)
+
+    if last is None:
+        last = pad
+
+    if current_size < desired_screen_length:
+        nadd = (desired_screen_length - current_size)
+        padding = (pad * (nadd - 1))
+        if align_right:
+            s = last + padding + s
+        else:
+            s = s + padding + last
+
+    # if debug_padding:
+    #     if current_size > desired_screen_length:
+    #         T = '(cut)'
+    #         s = s[:desired_screen_length - len(T)] + T
+
+    return s
